local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceConfig = LibStub("AceConfig-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("MountRandomMount")

local MRM_OptionsTable = {
	name = "MountRandomMount",
	handler = MountRandomMount,
	type = "group",
	args = {--MountRandomMount
		desc = {
			order = 1,
			name = L['MRMCommands'],
			fontSize = "medium",
			type = "description"
		},
	}
}

--POPULATE GROUND MOUNTS OPTIONS
function MountRandomMount:GroundFrame()
	local optionTable = {}
	local allmounts = {}
	local allMountsOrder = {}
	local EnableMounts = {}
	local sources, mounts, k, v, j

	for sources, mounts in pairs(MountRandomMount_MountDataList["ground"]) do
		local optionTableMount = {}
		table.foreach(mounts, function(k,v)
			if GetSpellInfo(v) ~= nil then
				local spellName = GetSpellInfo(v)
				optionTableMount[spellName..v] = {
					order = k,
					name = spellName,
					type = "toggle",
					hidden = function()
						if playerMountArray[v] == nil then
							return true
						else
							return false
						end
					end,
					set = function(info,val)
						local insert = true
						for iSaved, vSaved in pairs(MountRandomMount_SavedTable['ground']) do
							if vSaved == v then
								print(spellName.." "..L['MRMLandRemoved'])
								table.remove(MountRandomMount_SavedTable['ground'], iSaved)
								insert = false
								break
							end
						end
						if insert then
							print(spellName.." "..L['MRMLandAdded'])
							table.insert(MountRandomMount_SavedTable['ground'], v)
						end
					end,
					get = function(info)
						for iSaved, vSaved in pairs(MountRandomMount_SavedTable['ground']) do
							if vSaved == v then
								return true
							end
						end
						return false
					end
				}
				table.insert(allMountsOrder, spellName..v)
				allmounts[spellName..v] = optionTableMount[spellName..v]

				for iSaved, vSaved in pairs(_G['MountRandomMount_SavedTable']['ground']) do
					if vSaved == v then
						EnableMounts[spellName..v] = optionTableMount[spellName..v]
					end
				end
			else
				--print('The mount '..v..' does not exist')
			end
		end)
		if i ~= 0 then
			optionTable[sources] = {
				name = L[sources],
				order = 2,
				type = "group",
				handler = MountRandomMount,
				childGroups = "tab",
				args = optionTableMount
			}
		end
	end

	table.sort(allMountsOrder)
	j=1
	table.foreach(allMountsOrder, function(k,v)
		allmounts[v].order = j
		j=j+1
	end)

	optionTable['All'] = {
		name = L['All'],
		order = 1,
		type = "group",
		handler = MountRandomMount,
		childGroups = "tab",
		args = allmounts
	}

	optionTable['Enabled'] = {
		name = L['Enabled'],
		desc = L['This panel is only updated when you reload the UI or at loggin.'],
		order = 0,
		type = "group",
		handler = _G['MountRandomMount'],
		childGroups = "tab",
		args = EnableMounts
	}

	return {
		name = L["Ground Mount"],
		handler = MountRandomMount,
		type = "group",
		args = optionTable
	}
end

--POPULATE FLYING MOUNTS OPTIONS
function MountRandomMount:FlyingFrame()
	local optionTable = {}
	local allmounts = {}
	local allMountsOrder = {}
	local EnableMounts = {}
	local sources, mounts, k, v, j, i

	for sources, mounts in pairs(MountRandomMount_MountDataList["flying"]) do
		local optionTableMount = {}
		table.foreach(mounts, function(k,v)
			if GetSpellInfo(v) ~= nil then
				local spellName = GetSpellInfo(v)
				optionTableMount[spellName..v] = {
					order = k,
					name = spellName,
					type = "toggle",
					hidden = function()
						if _G['playerMountArray'][v] == nil then
							return true
						else
							return false
						end
					end,
					set = function(info,val)
						local insert = true
						for iSaved, vSaved in pairs(_G['MountRandomMount_SavedTable']['flying']) do
							if vSaved == v then
								print(spellName.." "..L['MRMFlyingRemoved'])
								table.remove(_G['MountRandomMount_SavedTable']['flying'], iSaved)
								insert = false
								break
							end
						end
						if insert then
							print(spellName.." "..L['MRMFlyingAdded'])
							table.insert(_G['MountRandomMount_SavedTable']['flying'], v)
						end
					end,
					get = function(info)
						for iSaved, vSaved in pairs(_G['MountRandomMount_SavedTable']['flying']) do
							if vSaved == v then
								return true
							end
						end
						return false
					end
				}
				table.insert(allMountsOrder, spellName..v)
				allmounts[spellName..v] = optionTableMount[spellName..v]
				for iSaved, vSaved in pairs(_G['MountRandomMount_SavedTable']['flying']) do
					if vSaved == v then
						EnableMounts[spellName..v] = optionTableMount[spellName..v]
					end
				end
			else
				--print('The mount '..v..' does not exist')
			end
		end)
		if i ~= 0 then
			optionTable[sources] = {
				name = L[sources],
				order = 2,
				type = "group",
				handler = _G['MountRandomMount'],
				childGroups = "tab",
				args = optionTableMount
			}
		end
	end

	table.sort(allMountsOrder)
	j=1
	table.foreach(allMountsOrder, function(k,v)
		allmounts[v].order = j
		j=j+1
	end)

	optionTable['All'] = {
		name = L['All'],
		order = 1,
		type = "group",
		handler = _G['MountRandomMount'],
		childGroups = "tab",
		args = allmounts
	}

	optionTable['Enabled'] = {
		name = L['Enabled'],
		desc = L['This panel is only updated when you reload the UI or at loggin.'],
		order = 0,
		type = "group",
		handler = _G['MountRandomMount'],
		childGroups = "tab",
		args = EnableMounts
	}

	return {
		name = L["Flying Mount"],
		handler = _G['MountRandomMount'],
		type = "group",
		args = optionTable
	}
end

function MountRandomMount:ResetOptions()
	MountRandomMount_SavedTable['ground'] = {}
	MountRandomMount_SavedTable['flying'] = {}
end

function MountRandomMount:PlayerMounts()
	_G['playerMountArray'] = {}
	local mountcount = GetNumCompanions("MOUNT")
	local i
	for i = 1, mountcount do
		local _, creatureName, creatureSpellID = GetCompanionInfo("MOUNT", i)
		playerMountArray[creatureSpellID] = creatureName
	end

	_G['MountRandomMount_SeaTurtleHave'] = true
	if playerMountArray[98718] ~= nil then
		_G['MountRandomMount_SeaTurtleID'] = 98718
	elseif playerMountArray[64731] ~= nil then
		_G['MountRandomMount_SeaTurtleID'] = 64731
	else
		_G['MountRandomMount_SeaTurtleHave'] = false
	end

	if playerMountArray[75207] ~= nil then
		_G['MountRandomMount_SeaHorseID'] = 98718
		_G['MountRandomMount_SeaHorseHave'] = true
	else
		_G['MountRandomMount_SeaHorseHave'] = false
	end
end

function MountRandomMount:PLAYER_LOGIN()
	MountRandomMount:MountDataListFill()
	if(MountRandomMount_SavedTable['ground'] == nil)then
		MountRandomMount_SavedTable['ground'] = {}
	end
	if(MountRandomMount_SavedTable['flying'] == nil)then
		MountRandomMount_SavedTable['flying'] = {}
	end

	MountRandomMount:PlayerMounts()

	AceConfig:RegisterOptionsTable("MountRandomMount", MRM_OptionsTable)
	AceConfig:RegisterOptionsTable("MountRandomMount_Ground", MountRandomMount:GroundFrame())
	AceConfig:RegisterOptionsTable("MountRandomMount_Flying", MountRandomMount:FlyingFrame())

	local SetDefaultOpttion = AceConfigDialog:AddToBlizOptions("MountRandomMount","MountRandomMount")
	SetDefaultOpttion.default = function() MountRandomMount:ResetOptions() end
	SetDefaultOpttion = AceConfigDialog:AddToBlizOptions("MountRandomMount_Ground",L["Ground Mount"],"MountRandomMount")
	SetDefaultOpttion.default = function() MountRandomMount:ResetOptions() end
	SetDefaultOpttion = AceConfigDialog:AddToBlizOptions("MountRandomMount_Flying",L["Flying Mount"],"MountRandomMount")
	SetDefaultOpttion.default = function() MountRandomMount:ResetOptions() end
end

function MountRandomMount:COMPANION_LEARNED()
	MountRandomMount:PlayerMounts()
end

function MountRandomMount:OnInitialize()
	MountRandomMount:RegisterEvent("PLAYER_LOGIN")
	MountRandomMount:RegisterEvent("COMPANION_LEARNED")
end